<?php

namespace Bixo\Niche\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Niche\Forms\Niche as NicheForm;
use Bixo\Niche\Http\Requests\NicheResourceRequest;
use Bixo\Niche\Http\Resources\NicheResource;
use Bixo\Niche\Http\Resources\NichesCollection;
use Bixo\Niche\Models\Niche;
use Bixo\Niche\Actions\NicheAction;
use Bixo\Niche\Actions\NicheActions;


/**
 * Resource controller class for niche.
 */
class NicheResourceController extends BaseController
{

    /**
     * Initialize niche resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = NicheForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.niche.modules'), 'niche', guard_url('niche'));
            return $next($request);
        });
    }

    /**
     * Display a list of niche.
     *
     * @return Response
     */
    public function index(NicheResourceRequest $request)
    {
        $request = $request->all();
        $page = NicheActions::run('paginate', $request);

        $data = new NichesCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('niche::niche.names'))
            ->view('niche::niche.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display niche.
     *
     * @param Request $request
     * @param Model   $niche
     *
     * @return Response
     */
    public function show(NicheResourceRequest $request, Niche $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new NicheResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('niche::niche.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('niche::niche.show')
            ->output();
    }

    /**
     * Show the form for creating a new niche.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(NicheResourceRequest $request, Niche $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new NicheResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('niche::niche.name'))
            ->view('niche::niche.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new niche.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(NicheResourceRequest $request, Niche $model)
    {
        try {
            $request = $request->all();
            $model = NicheAction::run('store', $model, $request);
            $data = new NicheResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('niche::niche.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('niche/niche/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/niche/niche'))
                ->redirect();
        }

    }

    /**
     * Show niche for editing.
     *
     * @param Request $request
     * @param Model   $niche
     *
     * @return Response
     */
    public function edit(NicheResourceRequest $request, Niche $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new NicheResource($model);
        // return view('niche::niche.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('niche::niche.name'))
            ->view('niche::niche.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the niche.
     *
     * @param Request $request
     * @param Model   $niche
     *
     * @return Response
     */
    public function update(NicheResourceRequest $request, Niche $model)
    {
        try {
            $request = $request->all();
            $model = NicheAction::run('update', $model, $request);
            $data = new NicheResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('niche::niche.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('niche/niche/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('niche/niche/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the niche.
     *
     * @param Model   $niche
     *
     * @return Response
     */
    public function destroy(NicheResourceRequest $request, Niche $model)
    {
        try {

            $request = $request->all();
            $model = NicheAction::run('destroy', $model, $request);
            $data = new NicheResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('niche::niche.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('niche/niche/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('niche/niche/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
